searchData={"content_type":"text/markdown","items":[{"doc":"Ewebmachine is a full rewrite with clean DSL and plug integration\nbased on Webmachine from basho. This version is not backward compatible with\nthe previous one that was only a thin wrapper around webmachine, use the branch\n1.0-legacy to use the old one.\n\n\n\n\n\nThe principle is to go through the [HTTP decision tree](http_diagram.png)\nand make decisions according to response of some callbacks called \"handlers\".\n\nTo do that, the library gives you 5 plugs and 2 plug pipeline builders :\n\n- `Ewebmachine.Plug.Run` go through the HTTP decision tree and fill\n  the `conn` response according to it\n- `Ewebmachine.Plug.Send` is used to send a conn set with `Ewebmachine.Plug.Run`\n- `Ewebmachine.Plug.Debug` gives you a debugging web UI to see the\n  HTTP decision path taken by each request.\n- `Ewebmachine.Plug.ErrorAsException` take a conn with a response set but not\n  send, and throw an exception is the status code is an exception\n- `Ewebmachine.Plug.ErrorAsForward` take a conn with a response set but not\n  send, and forward it changing the request to `GET /error/pattern/:status`\n- `Ewebmachine.Builder.Handlers` gives you helpers macros and a\n  `:add_handler` plug to add `handlers` as defined  in\n  `Ewebmachine.Handlers` to your conn, and set the initial user state.\n- `Ewebmachine.Builder.Resources` gives you a `resource` macro to\n  define at the same time an `Ewebmachine.Builder.Handlers` and the\n  matching spec to use it, and a plug `:resource_match` to do the\n  match and execute the associated plug. The macro `resources_plugs` helps you\n  to define commong plug pipeling","ref":"Ewebmachine.html","title":"Ewebmachine","type":"module"},{"doc":"```elixir\ndefmodule MyJSONApi do \n  use Ewebmachine.Builder.Handlers\n  plug :cors\n  plug :add_handlers, init: %{}\n\n  content_types_provided do: [\"application/json\": :to_json]\n  defh to_json, do: Poison.encode!(state[:json_obj])\n\n  defp cors(conn,_), do: \n    put_resp_header(conn,\"Access-Control-Allow-Origin\",\"*\")\nend\n\n\ndefmodule ErrorRoutes do\n  use Ewebmachine.Builder.Resources ; resources_plugs\n  resource \"/error/:status\" do %{s: elem(Integer.parse(status),0)} after \n    content_types_provided do: ['text/html': :to_html, 'application/json': :to_json]\n    defh to_html, do: \"  Error ! : '#{Ewebmachine.Core.Utils.http_label(state.s)}' \"\n    defh to_json, do: ~s/{\"error\": #{state.s}, \"label\": \"#{Ewebmachine.Core.Utils.http_label(state.s)}\"}/\n    finish_request do: {:halt,state.s}\n  end\nend\n\ndefmodule FullApi do\n  use Ewebmachine.Builder.Resources\n  if Mix.env == :dev, do: plug Ewebmachine.Plug.Debug\n  resources_plugs error_forwarding: \"/error/:status\", nomatch_404: true\n  plug ErrorRoutes\n\n  resource \"/hello/:name\" do %{name: name} after \n    content_types_provided do: ['application/xml': :to_xml]\n    defh to_xml, do: \"  #{state.name} \"\n  end\n\n  resource \"/hello/json/:name\" do %{name: name} after \n    plug MyJSONApi #this is also a plug pipeline\n    allowed_methods do: [\"GET\",\"DELETE\"]\n    resource_exists do: pass((user=DB.get(state.name)) !== nil, json_obj: user)\n    delete_resource do: DB.delete(state.name)\n  end\n\n  resource \"/static/*path\" do %{path: Enum.join(path,\"/\")} after\n    resource_exists do:\n      File.regular?(path state.path)\n    content_types_provided do:\n      [{state.path|>Plug.MIME.path|>default_plain,:to_content}]\n    defh to_content, do:\n      File.stream!(path(state.path),[],300_000_000)\n    defp path(relative), do: \"#{:code.priv_dir :ewebmachine_example}/web/#{relative}\"\n    defp default_plain(\"application/octet-stream\"), do: \"text/plain\"\n    defp default_plain(type), do: type\n  end\nend\n```","ref":"Ewebmachine.html#module-example-usage","title":"Example usage - Ewebmachine","type":"module"},{"doc":"Go to `/wm_debug` to see precedent requests and debug there HTTP\ndecision path. The debug UI can be updated automatically on the\nrequests.\n\n![Debug UI example](debug_ui.png)","ref":"Ewebmachine.html#module-debug-ui","title":"Debug UI - Ewebmachine","type":"module"},{"doc":"Create a simple supervision tree with only the Cowboy server adapter spec.\n\n```elixir\ndefmodule MyApp do\n  use Application\n  def start(_type, _args), do:\n    Supervisor.start_link([\n        Plug.Adapters.Cowboy.child_spec(:http,FullApi,[], port: 4000)\n      ], strategy: :one_for_one)\nend\n```\n\nAnd add it as your application entry point in your `mix.exs`\n\n```elixir\ndef application do\n  [applications: [:logger,:ewebmachine,:cowboy], mod: {MyApp,[]}]\nend\ndefp deps, do:\n  [{:ewebmachine, \"2.0.0\"}, {:cowboy, \"~> 1.0\"}]\n```\n\n# CONTRIBUTING\n\nHi, and thank you for wanting to contribute.\nPlease refer to the centralized informations available at: https://github.com/kbrw#contributing","ref":"Ewebmachine.html#module-use-cowboy-to-serve-the-plug","title":"Use Cowboy to serve the plug - Ewebmachine","type":"module"},{"doc":"Set :resp_redirect to `true`","ref":"Ewebmachine.html#do_redirect/1","title":"Ewebmachine.do_redirect/1","type":"function"},{"doc":"Fetch request body\n\nOptions: \n* max_length: maximum bytes to fetch (default: 1_000_000)","ref":"Ewebmachine.html#fetch_req_body/2","title":"Ewebmachine.fetch_req_body/2","type":"function"},{"doc":"Returns request body from request (requires fetching body first)","ref":"Ewebmachine.html#req_body/1","title":"Ewebmachine.req_body/1","type":"function"},{"doc":"`use` this  module will `use Plug.Builder` (so a plug pipeline\ndescribed with the `plug module_or_function_plug` macro), but gives\nyou an `:add_handler` local function plug which adds to the conn\nthe locally defined ewebmachine handlers (see `Ewebmachine.Handlers`).\n\nSo : \n\n- Construct your automate decision handler through multiple `:add_handler` plugs\n- Pipe the plug `Ewebmachine.Plug.Run` to run the HTTP automate which\n  will call these handlers to take decisions. \n- Pipe the plug `Ewebmachine.Plug.Send` to send and halt any conn previsously passed\n  through an automate run.\n\nTo define handlers, use the following helpers :\n\n- the handler specific macros (like `Ewebmachine.Builder.Handlers.resource_exists/1`)\n- the macro `defh/2` to define any helpers, usefull for body\n  producing handlers or to have multiple function clauses \n- in handler implementation `conn` and `state` binding are available\n- the response of the handler implementation is wrapped, so that\n  returning `:my_response` is the same as returning `{:my_response,conn,state}`\n\nBelow a full example :\n\n```\ndefmodule MyJSONApi do \n  use Ewebmachine.Builder.Handlers\n  plug :cors\n  plug :add_handlers, init: %{}\n\n  content_types_provided do: [\"application/json\": :to_json]\n  defh to_json, do: Poison.encode!(state[:json_obj])\n\n  defp cors(conn,_), do: \n    put_resp_header(conn,\"Access-Control-Allow-Origin\",\"*\")\nend\n\ndefmodule GetUser do \n  use Ewebmachine.Builder.Handlers\n  plug MyJSONApi\n  plug :add_handlers\n  plug Ewebmachine.Plug.Run\n  plug Ewebmachine.Plug.Send\n  resource_exists do:\n    pass( !is_nil(user=DB.User.get(conn.params[\"q\"])), json_obj: user)\nend\ndefmodule GetOrder do \n  use Ewebmachine.Builder.Handlers\n  plug MyJSONApi\n  plug :add_handlers\n  plug Ewebmachine.Plug.Run\n  plug Ewebmachine.Plug.Send\n  resource_exists do:\n    pass(!is_nil(order=DB.Order.get(conn.params[\"q\"])), json_obj: order)\nend\n\ndefmodule API do\n  use Plug.Router\n  plug :match \n  plug :dispatch\n\n  get \"/get/user\", do: GetUser.call(conn,[])\n  get \"/get/order\", do: GetOrder.call(conn,[])\n  end\n```","ref":"Ewebmachine.Builder.Handlers.html","title":"Ewebmachine.Builder.Handlers","type":"module"},{"doc":"see `Ewebmachine.Handlers.allow_missing_post/2`","ref":"Ewebmachine.Builder.Handlers.html#allow_missing_post/1","title":"Ewebmachine.Builder.Handlers.allow_missing_post/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.allowed_methods/2`","ref":"Ewebmachine.Builder.Handlers.html#allowed_methods/1","title":"Ewebmachine.Builder.Handlers.allowed_methods/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.base_uri/2`","ref":"Ewebmachine.Builder.Handlers.html#base_uri/1","title":"Ewebmachine.Builder.Handlers.base_uri/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.charsets_provided/2`","ref":"Ewebmachine.Builder.Handlers.html#charsets_provided/1","title":"Ewebmachine.Builder.Handlers.charsets_provided/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.content_types_accepted/2`","ref":"Ewebmachine.Builder.Handlers.html#content_types_accepted/1","title":"Ewebmachine.Builder.Handlers.content_types_accepted/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.content_types_provided/2`","ref":"Ewebmachine.Builder.Handlers.html#content_types_provided/1","title":"Ewebmachine.Builder.Handlers.content_types_provided/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.create_path/2`","ref":"Ewebmachine.Builder.Handlers.html#create_path/1","title":"Ewebmachine.Builder.Handlers.create_path/1","type":"macro"},{"doc":"define a resource handler function as described at\n`Ewebmachine.Handlers`.\n\nSince there is a specific macro in this module for each handler,\nthis macro is useful : \n\n- to define body producing and body processing handlers (the one\n  referenced in the response of `Ewebmachine.Handlers.content_types_provided/2` or \n  `Ewebmachine.Handlers.content_types_accepted/2`)\n- to explicitly take the `conn` and the `state` parameter, which\n  allows you to add guards and pattern matching for instance to\n  define multiple clauses for the handler\n\n```\ndefh to_html, do: \"hello you\"\ndefh from_json, do: pass(:ok, json: Poison.decode!(read_body conn))\n```\n\n```\ndefh resources_exists(conn,%{obj: obj}) when obj !== nil, do: true\ndefh resources_exists(conn,_), do: false\n```","ref":"Ewebmachine.Builder.Handlers.html#defh/2","title":"Ewebmachine.Builder.Handlers.defh/2","type":"macro"},{"doc":"see `Ewebmachine.Handlers.delete_completed/2`","ref":"Ewebmachine.Builder.Handlers.html#delete_completed/1","title":"Ewebmachine.Builder.Handlers.delete_completed/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.delete_resource/2`","ref":"Ewebmachine.Builder.Handlers.html#delete_resource/1","title":"Ewebmachine.Builder.Handlers.delete_resource/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.encodings_provided/2`","ref":"Ewebmachine.Builder.Handlers.html#encodings_provided/1","title":"Ewebmachine.Builder.Handlers.encodings_provided/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.expires/2`","ref":"Ewebmachine.Builder.Handlers.html#expires/1","title":"Ewebmachine.Builder.Handlers.expires/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.finish_request/2`","ref":"Ewebmachine.Builder.Handlers.html#finish_request/1","title":"Ewebmachine.Builder.Handlers.finish_request/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.forbidden/2`","ref":"Ewebmachine.Builder.Handlers.html#forbidden/1","title":"Ewebmachine.Builder.Handlers.forbidden/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.generate_etag/2`","ref":"Ewebmachine.Builder.Handlers.html#generate_etag/1","title":"Ewebmachine.Builder.Handlers.generate_etag/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.is_authorized/2`","ref":"Ewebmachine.Builder.Handlers.html#is_authorized/1","title":"Ewebmachine.Builder.Handlers.is_authorized/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.is_conflict/2`","ref":"Ewebmachine.Builder.Handlers.html#is_conflict/1","title":"Ewebmachine.Builder.Handlers.is_conflict/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.known_content_type/2`","ref":"Ewebmachine.Builder.Handlers.html#known_content_type/1","title":"Ewebmachine.Builder.Handlers.known_content_type/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.known_methods/2`","ref":"Ewebmachine.Builder.Handlers.html#known_methods/1","title":"Ewebmachine.Builder.Handlers.known_methods/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.last_modified/2`","ref":"Ewebmachine.Builder.Handlers.html#last_modified/1","title":"Ewebmachine.Builder.Handlers.last_modified/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.malformed_request/2`","ref":"Ewebmachine.Builder.Handlers.html#malformed_request/1","title":"Ewebmachine.Builder.Handlers.malformed_request/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.moved_permanently/2`","ref":"Ewebmachine.Builder.Handlers.html#moved_permanently/1","title":"Ewebmachine.Builder.Handlers.moved_permanently/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.moved_temporarily/2`","ref":"Ewebmachine.Builder.Handlers.html#moved_temporarily/1","title":"Ewebmachine.Builder.Handlers.moved_temporarily/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.multiple_choices/2`","ref":"Ewebmachine.Builder.Handlers.html#multiple_choices/1","title":"Ewebmachine.Builder.Handlers.multiple_choices/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.options/2`","ref":"Ewebmachine.Builder.Handlers.html#options/1","title":"Ewebmachine.Builder.Handlers.options/1","type":"macro"},{"doc":"Shortcut macro for :\n{response,var!(conn),Enum.into(update_state,var!(state))}\n\nuse it if your handler wants to add some value to a collectable\nstate (a map for instance), but using default \"conn\" current\nbinding.\n\nfor instance a resources_exists implementation \"caching\" the result\nin the state could be :\n\n    pass (user=DB.get(state.id)) != nil, current_user: user\n    # same as returning :\n    {true,conn,%{id: \"arnaud\", current_user: %User{id: \"arnaud\"}}}","ref":"Ewebmachine.Builder.Handlers.html#pass/2","title":"Ewebmachine.Builder.Handlers.pass/2","type":"macro"},{"doc":"see `Ewebmachine.Handlers.ping/2`","ref":"Ewebmachine.Builder.Handlers.html#ping/1","title":"Ewebmachine.Builder.Handlers.ping/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.post_is_create/2`","ref":"Ewebmachine.Builder.Handlers.html#post_is_create/1","title":"Ewebmachine.Builder.Handlers.post_is_create/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.previously_existed/2`","ref":"Ewebmachine.Builder.Handlers.html#previously_existed/1","title":"Ewebmachine.Builder.Handlers.previously_existed/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.process_post/2`","ref":"Ewebmachine.Builder.Handlers.html#process_post/1","title":"Ewebmachine.Builder.Handlers.process_post/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.resource_exists/2`","ref":"Ewebmachine.Builder.Handlers.html#resource_exists/1","title":"Ewebmachine.Builder.Handlers.resource_exists/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.service_available/2`","ref":"Ewebmachine.Builder.Handlers.html#service_available/1","title":"Ewebmachine.Builder.Handlers.service_available/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.uri_too_long/2`","ref":"Ewebmachine.Builder.Handlers.html#uri_too_long/1","title":"Ewebmachine.Builder.Handlers.uri_too_long/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.valid_content_headers/2`","ref":"Ewebmachine.Builder.Handlers.html#valid_content_headers/1","title":"Ewebmachine.Builder.Handlers.valid_content_headers/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.valid_entity_length/2`","ref":"Ewebmachine.Builder.Handlers.html#valid_entity_length/1","title":"Ewebmachine.Builder.Handlers.valid_entity_length/1","type":"macro"},{"doc":"see `Ewebmachine.Handlers.variances/2`","ref":"Ewebmachine.Builder.Handlers.html#variances/1","title":"Ewebmachine.Builder.Handlers.variances/1","type":"macro"},{"doc":"`use` this  module will `use Plug.Builder` (so a plug pipeline\ndescribed with the `plug module_or_function_plug` macro), but gives\nyou a `:resource_match` local function plug which matches routes declared\nwith the `resource/2` macro and execute the plug defined by its body.\n\nSee `Ewebmachine.Builder.Handlers` documentation to see how to\ncontruct these modules (in the `after` block)\n\nBelow a full example :\n\n```\n\ndefmodule FullApi do\n  use Ewebmachine.Builder.Resources\n  if Mix.env == :dev, do: plug Ewebmachine.Plug.Debug\n  # pre plug, for instance you can put plugs defining common handlers\n  plug :resource_match\n  plug Ewebmachine.Plug.Run\n  # customize ewebmachine result, for instance make an error page handler plug\n  plug Ewebmachine.Plug.Send\n  # plug after that will be executed only if no ewebmachine resources has matched\n\n  resource \"/hello/:name\" do %{name: name} after\n    plug SomeAdditionnalPlug\n    content_types_provided do: ['application/xml': :to_xml]\n    defh to_xml, do: \"  #{state.name} \"\n  end\n\n  resource \"/*path\" do %{path: Enum.join(path,\"/\")} after\n    resource_exists do:\n      File.regular?(path state.path)\n    content_types_provided do:\n      [{state.path|>Plug.MIME.path|>default_plain,:to_content}]\n    defh to_content, do:\n      File.stream!(path(state.path),[],300_000_000)\n    defp path(relative), do: \"#{:code.priv_dir :ewebmachine_example}/web/#{relative}\"\n    defp default_plain(\"application/octet-stream\"), do: \"text/plain\"\n    defp default_plain(type), do: type\n  end\nend\n```","ref":"Ewebmachine.Builder.Resources.html","title":"Ewebmachine.Builder.Resources","type":"module"},{"doc":"As the most common use case is to match resources, run the webmachine\nautomate, then set a 404 if no resource match, then handle error code, then\nsend the response, the `resources_plugs/1` macro allows you to do that.\n\nFor example, if you want to convert all HTTP errors as Exceptions, and\nconsider that all path must be handled and so any non matching path should\nreturn a 404 :\n\n    resources_plugs error_as_exception: true, nomatch_404: true\n\nis equivalent to\n\n    plug :resource_match\n    plug Ewebmachine.Plug.Run\n    plug :wm_notset_404\n    plug Ewebmachine.Plug.ErrorAsException\n    plug Ewebmachine.Plug.Send\n\n    defp wm_notset_404(%{state: :unset}=conn,_), do: resp(conn,404,\"\")\n    defp wm_notset_404(conn,_), do: conn\n\nAnother example, following plugs must handle non matching paths and errors\nshould be converted into `GET /error/:status` that must be handled by\nfollowing plugs :\n\n    resources_plugs error_forwarding: \"/error/:status\"\n\nis equivalent to\n\n    plug :resource_match\n    plug Ewebmachine.Plug.Run\n    plug Ewebmachine.Plug.ErrorAsForward, forward_pattern: \"/error/:status\"\n    plug Ewebmachine.Plug.Send","ref":"Ewebmachine.Builder.Resources.html#module-common-plugs-macro-helper","title":"Common Plugs macro helper - Ewebmachine.Builder.Resources","type":"module"},{"doc":"","ref":"Ewebmachine.Builder.Resources.html#resource/2","title":"Ewebmachine.Builder.Resources.resource/2","type":"macro"},{"doc":"Create a webmachine handler plug and use it on `:resource_match` when path matches\n\n- the route will be the matching spec (see Plug.Router.match, string spec only)\n- do_block will be called on match (so matching bindings will be\n  available) and should return the initial state\n- after_block will be the webmachine handler plug module body\n  (wrapped with `use Ewebmachine.Builder.Handlers` and `plug\n  :add_handlers` to clean the declaration.\n\n```\nresource \"/my/route/:commaid\" do\n  id = string.split(commaid,\",\")\n  %{foo: id}\nafter\n  plug someadditionnalplug\n  resource_exists do: state.id == [\"hello\"]\nend\n\nresource ShortenedRouteName, \"/my/route/that/would/generate/a/long/module/name/:commaid\" do\n  id = String.split(commaid,\",\")\n  %{foo: id}\nafter\n  plug SomeAdditionnalPlug\n  resource_exists do: state.id == [\"hello\"]\nend\n```","ref":"Ewebmachine.Builder.Resources.html#resource/3","title":"Ewebmachine.Builder.Resources.resource/3","type":"macro"},{"doc":"","ref":"Ewebmachine.Builder.Resources.html#resource_quote/4","title":"Ewebmachine.Builder.Resources.resource_quote/4","type":"function"},{"doc":"","ref":"Ewebmachine.Builder.Resources.html#resources_plugs/1","title":"Ewebmachine.Builder.Resources.resources_plugs/1","type":"macro"},{"doc":"HTTP utility module","ref":"Ewebmachine.Core.Utils.html","title":"Ewebmachine.Core.Utils","type":"module"},{"doc":"HTTP Charset negociation, get the charset to use from : \n\n- `acc_char_hdr`, the HTTP header `Accept-Charset`\n- `charsets`, the list of supported charsets","ref":"Ewebmachine.Core.Utils.html#choose_charset/2","title":"Ewebmachine.Core.Utils.choose_charset/2","type":"function"},{"doc":"HTTP Encoding negociation, get the encoding to use from : \n\n- `acc_enc_hdr`, the HTTP header `Accept-Encoding`\n- `encs`, the list of supported encoding","ref":"Ewebmachine.Core.Utils.html#choose_encoding/2","title":"Ewebmachine.Core.Utils.choose_encoding/2","type":"function"},{"doc":"HTTP Content negociation, get the content type to send from : \n\n- `accept_header`, the HTTP header `Accept`\n- `ct_provided`, the list of provided content types","ref":"Ewebmachine.Core.Utils.html#choose_media_type/2","title":"Ewebmachine.Core.Utils.choose_media_type/2","type":"function"},{"doc":"Convert rfc1123 or rfc850 to :calendar dates","ref":"Ewebmachine.Core.Utils.html#convert_request_date/1","title":"Ewebmachine.Core.Utils.convert_request_date/1","type":"function"},{"doc":"Format a `norm_content_type` into an HTTP content type header","ref":"Ewebmachine.Core.Utils.html#format_mtype/1","title":"Ewebmachine.Core.Utils.format_mtype/1","type":"function"},{"doc":"Match normalized media types accepting a partial match (wildcard or\nincomplete params)","ref":"Ewebmachine.Core.Utils.html#fuzzy_mt_match/2","title":"Ewebmachine.Core.Utils.fuzzy_mt_match/2","type":"function"},{"doc":"Get HTTP status label from HTTP code","ref":"Ewebmachine.Core.Utils.html#http_label/1","title":"Ewebmachine.Core.Utils.http_label/1","type":"function"},{"doc":"Convert any content type representation (see spec) into a `norm_content_type`","ref":"Ewebmachine.Core.Utils.html#normalize_mtype/1","title":"Ewebmachine.Core.Utils.normalize_mtype/1","type":"function"},{"doc":"Remove quotes from HTTP quoted string","ref":"Ewebmachine.Core.Utils.html#quoted_string/1","title":"Ewebmachine.Core.Utils.quoted_string/1","type":"function"},{"doc":"Convert a calendar date to a rfc1123 date string","ref":"Ewebmachine.Core.Utils.html#rfc1123_date/1","title":"Ewebmachine.Core.Utils.rfc1123_date/1","type":"function"},{"doc":"Get the string list from a comma separated list of HTTP quoted strings","ref":"Ewebmachine.Core.Utils.html#split_quoted_strings/1","title":"Ewebmachine.Core.Utils.split_quoted_strings/1","type":"function"},{"doc":"","ref":"Ewebmachine.Core.Utils.html#t:norm_content_type/0","title":"Ewebmachine.Core.Utils.norm_content_type/0","type":"type"},{"doc":"","ref":"Ewebmachine.Events.html","title":"Ewebmachine.Events","type":"module"},{"doc":"","ref":"Ewebmachine.Events.html#child_spec/1","title":"Ewebmachine.Events.child_spec/1","type":"function"},{"doc":"","ref":"Ewebmachine.Events.html#dispatch/1","title":"Ewebmachine.Events.dispatch/1","type":"function"},{"doc":"","ref":"Ewebmachine.Events.html#stream_chunks/1","title":"Ewebmachine.Events.stream_chunks/1","type":"function"},{"doc":"Implement the functions described below to make decisions in the \n[HTTP decision tree](http_diagram.png) :\n\n- `service_available/2`\n- `resource_exists/2`\n- `is_authorized/2`\n- `forbidden/2`\n- `allow_missing_post/2`\n- `malformed_request/2`\n- `uri_too_long/2`\n- `known_content_type/2`\n- `valid_content_headers/2`\n- `valid_entity_length/2`\n- `options/2`\n- `allowed_methods/2`\n- `known_methods/2`\n- `content_types_provided/2`\n- `content_types_accepted/2`\n- `delete_resource/2`\n- `delete_completed/2`\n- `post_is_create/2`\n- `create_path/2`\n- `base_uri/2`\n- `process_post/2`\n- `language_available/2`\n- `charsets_provided/2`\n- `encodings_provided/2`\n- `variances/2`\n- `is_conflict/2`\n- `multiple_choices/2`\n- `previously_existed/2`\n- `moved_permanently/2`\n- `moved_temporarily/2`\n- `last_modified/2`\n- `expires/2`\n- `generate_etag/2`\n- `validate_content_checksum/2`\n- `ping/2`\n- Body-producing function, see `to_html/2` (but any function name can be\n  used, as referenced by `content_types_provided/2`\n- POST/PUT processing function, see `from_json/2` (but any function name can be\n  used, as referenced by `content_types_accepted/2`\n\nAll the handlers have the same signature :\n\n```\n(conn :: Plug.Conn.t,state :: any)->{response :: any | {:halt,200..599},conn :: Plug.Conn.t, state :: any}\n```\n\nwhere every handler implementation : \n\n- can change or halt the plug `conn` passed as argument\n- can change the user state object passed from on handler to another in its arguments\n- returns something which will make decision in the HTTP decision tree (see\n  documentation of functions in this module to see expected results and effects)\n- can return `{:halt,200..599}` to end the ewebmachine automate execution,\n  but do not `halt` the `conn`, so the plug pipeline can continue.\n\nSo each handler implementation is actually a \"plug\" returning a response giving\ninformation allowing to make the good response code and path in the HTTP\nspecification.","ref":"Ewebmachine.Handlers.html","title":"Ewebmachine.Handlers","type":"module"},{"doc":"The following modules will help you to construct these handlers and use them :\n\n- `Ewebmachine.Builder.Handlers` gives you macros and helpers to define the\n  handlers and automatically create the plug to add them to your `conn.private[:resource_handlers]`\n- `Ewebmachine.Plug.Run` run the HTTP decision tree executing the handler\n   implementations described in its `conn.private[:resource_handlers]`. The\n   initial user `state` is taken in `conn.private[:machine_init]`","ref":"Ewebmachine.Handlers.html#module-usage","title":"Usage ## - Ewebmachine.Handlers","type":"module"},{"doc":"If the resource accepts POST requests to nonexistent resources, then this should return `true`.\n\nDefault: `false`","ref":"Ewebmachine.Handlers.html#allow_missing_post/2","title":"Ewebmachine.Handlers.allow_missing_post/2","type":"function"},{"doc":"If a Method not in this list is requested, then a 405 Method Not\nAllowed will be sent. Note that these are all-caps Strings (binary).\n\nDefault: `[\"GET\", \"HEAD\"]`","ref":"Ewebmachine.Handlers.html#allowed_methods/2","title":"Ewebmachine.Handlers.allowed_methods/2","type":"function"},{"doc":"The base URI used in the location header on resource creation (when\n`post_is_create` is `true`), will be prepended to the `create_path`","ref":"Ewebmachine.Handlers.html#base_uri/2","title":"Ewebmachine.Handlers.base_uri/2","type":"function"},{"doc":"If this is anything other than the atom `:no_charset`, it must be a\n`{key,value}` Enumerable where `key` is the charset and `value` is a\ncallable function in the resource which will be called on the\nproduced body in a GET and ensure that it is in Charset.\n\nDefault: `:no_charset`","ref":"Ewebmachine.Handlers.html#charsets_provided/2","title":"Ewebmachine.Handlers.charsets_provided/2","type":"function"},{"doc":"This is used similarly to content_types_provided, except that it is\nfor incoming resource representations -- for example, PUT requests.\nHandler functions usually want to use `Plug.read_body(conn)` to\naccess the incoming request body.\n\nDefault: `[]`","ref":"Ewebmachine.Handlers.html#content_types_accepted/2","title":"Ewebmachine.Handlers.content_types_accepted/2","type":"function"},{"doc":"This should return a key value tuple enumerable where the key is\nthe content-type format and the value is an atom naming the\nfunction which can provide a resource representation in that media\ntype. Content negotiation is driven by this return value. For\nexample, if a client request includes an Accept header with a value\nthat does not appear as a first element in any of the return\ntuples, then a 406 Not Acceptable will be\nsent.\n\nDefault: `[{\"text/html\", to_html}]`","ref":"Ewebmachine.Handlers.html#content_types_provided/2","title":"Ewebmachine.Handlers.content_types_provided/2","type":"function"},{"doc":"This will be called on a POST request if `post_is_create` returns\ntrue. It is an error for this function not to produce a Path if\npost_is_create returns true. The Path returned should be a valid\nURI part.","ref":"Ewebmachine.Handlers.html#create_path/2","title":"Ewebmachine.Handlers.create_path/2","type":"function"},{"doc":"This is only called after a successful `delete_resource` call, and\nshould return `false` if the deletion was accepted but cannot yet be\nguaranteed to have finished.","ref":"Ewebmachine.Handlers.html#delete_completed/2","title":"Ewebmachine.Handlers.delete_completed/2","type":"function"},{"doc":"This is called when a DELETE request should be enacted, and should\nreturn `true` if the deletion succeeded.","ref":"Ewebmachine.Handlers.html#delete_resource/2","title":"Ewebmachine.Handlers.delete_resource/2","type":"function"},{"doc":"This must be a `{key,value}` Enumerable where `key` is a valid\ncontent encoding and `value` is a callable function in the resource\nwhich will be called on the produced body in a GET and ensure that\nit is so encoded. One useful setting is to have the function check\non method, and on GET requests return:\n\n```\n[identity: &(&1), gzip: &:zlib.gzip/1]\n```\n as this is all that is needed to support gzip content encoding.\n\n Default: `[{\"identity\", fn X-> X end}]`","ref":"Ewebmachine.Handlers.html#encodings_provided/2","title":"Ewebmachine.Handlers.encodings_provided/2","type":"function"},{"doc":"If not `nil`, set the expires header\n\nDefault: `nil`","ref":"Ewebmachine.Handlers.html#expires/2","title":"Ewebmachine.Handlers.expires/2","type":"function"},{"doc":"Last handler, always called. Response is ignored except if it is a `halt`.","ref":"Ewebmachine.Handlers.html#finish_request/2","title":"Ewebmachine.Handlers.finish_request/2","type":"function"},{"doc":"Returning true will result in 403 Forbidden.\n\nDefault: `false`","ref":"Ewebmachine.Handlers.html#forbidden/2","title":"Ewebmachine.Handlers.forbidden/2","type":"function"},{"doc":"Example POST/PUT processing function, function atom name must be referenced\nin `content_types_accepted/2`.\n\nIt will be called when the request is `PUT` or when the\n  request is `POST` and `post_is_create` returns true.","ref":"Ewebmachine.Handlers.html#from_json/2","title":"Ewebmachine.Handlers.from_json/2","type":"function"},{"doc":"If not `nil`, it will be used for the ETag header and\nfor comparison in conditional requests.\n\nDefault: `nil`","ref":"Ewebmachine.Handlers.html#generate_etag/2","title":"Ewebmachine.Handlers.generate_etag/2","type":"function"},{"doc":"If this returns anything other than `true`, the response will be\n`401 Unauthorized`. The return value will be used as the value in\nthe `WWW-Authenticate` header, for example `Basic\nrealm=\"Webmachine\"`.\n\nDefault: `true`","ref":"Ewebmachine.Handlers.html#is_authorized/2","title":"Ewebmachine.Handlers.is_authorized/2","type":"function"},{"doc":"If this returns `true`, the client will receive a 409 Conflict.\n\nDefault : `false`","ref":"Ewebmachine.Handlers.html#is_conflict/2","title":"Ewebmachine.Handlers.is_conflict/2","type":"function"},{"doc":"Returning false will result in 415 Unsupported Media Type.\n\nDefault: `true`","ref":"Ewebmachine.Handlers.html#known_content_type/2","title":"Ewebmachine.Handlers.known_content_type/2","type":"function"},{"doc":"Override the known methods accepted by your automate\n\nDefault: `[\"GET\", \"HEAD\", \"POST\", \"PUT\", \"DELETE\", \"TRACE\", \"CONNECT\", \"OPTIONS\"]`","ref":"Ewebmachine.Handlers.html#known_methods/2","title":"Ewebmachine.Handlers.known_methods/2","type":"function"},{"doc":"return false if language in\n`Plug.Conn.get_resp_header(conn,\"accept-language\")` is not\navailable.","ref":"Ewebmachine.Handlers.html#language_available/2","title":"Ewebmachine.Handlers.language_available/2","type":"function"},{"doc":"If this returns a `datetime()` (`{{day,month,year},{h,m,s}}`, it\nwill be used for the `Last-Modified` header and for comparison in\nconditional requests.\n\nDefault: `nil`","ref":"Ewebmachine.Handlers.html#last_modified/2","title":"Ewebmachine.Handlers.last_modified/2","type":"function"},{"doc":"Returning true will result in 400 Bad Request.\n\nDefault: `false`","ref":"Ewebmachine.Handlers.html#malformed_request/2","title":"Ewebmachine.Handlers.malformed_request/2","type":"function"},{"doc":"If this returns `{true, uri}`, the client will receive a `301 Moved\nPermanently` with `uri` in the Location header.\n\nDefault: `false`","ref":"Ewebmachine.Handlers.html#moved_permanently/2","title":"Ewebmachine.Handlers.moved_permanently/2","type":"function"},{"doc":"If this returns `{true, uri}`, the client will receive a `307\nTemporary Redirect` with `uri` in the Location header.\n\nDefault: `false`","ref":"Ewebmachine.Handlers.html#moved_temporarily/2","title":"Ewebmachine.Handlers.moved_temporarily/2","type":"function"},{"doc":"If this returns `true`, then it is assumed that multiple\nrepresentations of the response are possible and a single one\ncannot be automatically chosen, so a `300 Multiple Choices` will be\nsent instead of a `200 OK`.\n\nDefault: `false`","ref":"Ewebmachine.Handlers.html#multiple_choices/2","title":"Ewebmachine.Handlers.multiple_choices/2","type":"function"},{"doc":"If the OPTIONS method is supported and is used, the return value of\nthis function is expected to be a list of pairs representing header\nnames and values that should appear in the response.","ref":"Ewebmachine.Handlers.html#options/2","title":"Ewebmachine.Handlers.options/2","type":"function"},{"doc":"Must be present and returning `pong` to prove that handlers are\nwell linked to the automate","ref":"Ewebmachine.Handlers.html#ping/2","title":"Ewebmachine.Handlers.ping/2","type":"function"},{"doc":"If POST requests should be treated as a request to put content into\na (potentially new) resource as opposed to being a generic\nsubmission for processing, then this function should return true.\nIf it does return `true`, then `create_path` will be called and the\nrest of the request will be treated much like a PUT to the Path\nentry returned by that call.\n\nDefault: `false`","ref":"Ewebmachine.Handlers.html#post_is_create/2","title":"Ewebmachine.Handlers.post_is_create/2","type":"function"},{"doc":"If this returns `true`, the `moved_permanently` and `moved_temporarily`\ncallbacks will be invoked to determine whether the response should\nbe `301 Moved Permanently`, `307 Temporary Redirect`, or `410 Gone`.\n\nDefault: `false`","ref":"Ewebmachine.Handlers.html#previously_existed/2","title":"Ewebmachine.Handlers.previously_existed/2","type":"function"},{"doc":"If `post_is_create` returns `false`, then this will be called to\nprocess any POST requests. If it succeeds, it should return `true`.","ref":"Ewebmachine.Handlers.html#process_post/2","title":"Ewebmachine.Handlers.process_post/2","type":"function"},{"doc":"Returning non-true values will result in `404 Not Found`.\n\nDefault: `true`","ref":"Ewebmachine.Handlers.html#resource_exists/2","title":"Ewebmachine.Handlers.resource_exists/2","type":"function"},{"doc":"Returning non-true values will result in `503 Service Unavailable`.\n\nDefault: `true`","ref":"Ewebmachine.Handlers.html#service_available/2","title":"Ewebmachine.Handlers.service_available/2","type":"function"},{"doc":"Example body-producing function, function atom name must be referenced in `content_types_provided/2`.\n\n- If the result is an `Enumerable` of `iodata`, then the HTTP response will be\n  a chunk encoding response where each chunk on element of the enumeration.\n- If the result is an iodata, then it is used as the HTTP response body","ref":"Ewebmachine.Handlers.html#to_html/2","title":"Ewebmachine.Handlers.to_html/2","type":"function"},{"doc":"Returning true will result in 414 Request-URI Too Long.\n\nDefault: `false`","ref":"Ewebmachine.Handlers.html#uri_too_long/2","title":"Ewebmachine.Handlers.uri_too_long/2","type":"function"},{"doc":"Returning false will result in 501 Not Implemented.\n\nDefault: `true`","ref":"Ewebmachine.Handlers.html#valid_content_headers/2","title":"Ewebmachine.Handlers.valid_content_headers/2","type":"function"},{"doc":"Returning false will result in 413 Request Entity Too Large.\n\nDefault: `false`","ref":"Ewebmachine.Handlers.html#valid_entity_length/2","title":"Ewebmachine.Handlers.valid_entity_length/2","type":"function"},{"doc":"if `content-md5` header exists: \n- If `:not_validated`, test if input body validate `content-md5`,\n- if return `false`, then return a bad request\n\nUseful if content-md5 validation does not imply only raw md5 hash","ref":"Ewebmachine.Handlers.html#validate_content_checksum/2","title":"Ewebmachine.Handlers.validate_content_checksum/2","type":"function"},{"doc":"If this function is implemented, it should return a list of strings\nwith header names that should be included in a given response's\nVary header. The standard conneg headers (`Accept`, `Accept-Encoding`,\n`Accept-Charset`, `Accept-Language`) do not need to be specified here\nas Webmachine will add the correct elements of those automatically\ndepending on resource behavior.\n\nDefault : `[]`","ref":"Ewebmachine.Handlers.html#variances/2","title":"Ewebmachine.Handlers.variances/2","type":"function"},{"doc":"","ref":"Ewebmachine.Handlers.html#t:conn/0","title":"Ewebmachine.Handlers.conn/0","type":"type"},{"doc":"","ref":"Ewebmachine.Handlers.html#t:halt/0","title":"Ewebmachine.Handlers.halt/0","type":"type"},{"doc":"","ref":"Ewebmachine.Handlers.html#t:state/0","title":"Ewebmachine.Handlers.state/0","type":"type"},{"doc":"A ewebmachine debug UI at `/wm_debug`\n\nAdd it before `Ewebmachine.Plug.Run` in your plug pipeline when you\nwant debugging facilities.\n\n```\nif Mix.env == :dev, do: plug Ewebmachine.Plug.Debug\n```\n\nThen go to `http://youhost:yourport/wm_debug`, you will see the\nrequest list since the launch of your server. Click on any to get\nthe ewebmachine debugging UI. The list will be automatically\nupdated on new query.\n\nThe ewebmachine debugging UI \n\n- shows you the HTTP decision path taken by the request to the response. Every\n- the red decisions are the one where decisions differs from the\n  default one because of a handler implementation :\n  - click on them, then select any handler available in the right\n    tab to see the `conn`, `state` inputs of the handler and the\n    response.\n- The response and request right tab shows you the request and\n  result at the end of the ewebmachine run.\n- click on \"auto redirect on new query\" and at every request, your\n  browser will navigate to the debugging UI of the new request (you\n  can still use back/next to navigate through requests)\n\n![Debug UI example](debug_ui.png)","ref":"Ewebmachine.Plug.Debug.html","title":"Ewebmachine.Plug.Debug","type":"module"},{"doc":"","ref":"Ewebmachine.Plug.Debug.html#call/2","title":"Ewebmachine.Plug.Debug.call/2","type":"function"},{"doc":"","ref":"Ewebmachine.Plug.Debug.html#init/1","title":"Ewebmachine.Plug.Debug.init/1","type":"function"},{"doc":"This plug take an argument `forward_pattern` (default to `\"/error/:status\"`),\nand, when the current response status is an error, simply forward to a `GET`\nto the path defined by the pattern and this status.","ref":"Ewebmachine.Plug.ErrorAsForward.html","title":"Ewebmachine.Plug.ErrorAsForward","type":"module"},{"doc":"Plug passing your `conn` through the [HTTP decision tree](http_diagram.png)\n  to fill its status and response.\n\n  This plug does not send the HTTP result, instead the `conn`\n  result of this plug must be sent with the plug\n  `Ewebmachine.Plug.Send`. This is useful to customize the Ewebmachine result\n  after the run, for instance to customize the error body (void by default).\n  \n  - Decisions are make according to handlers set in `conn.private[:resource_handlers]` \n    (`%{handler_name: handler_module}`) where `handler_name` is one\n    of the handler function of `Ewebmachine.Handlers` and\n    `handler_module` is the module implementing it.\n  - Initial user state (second parameter of handler function) is\n    taken from `conn.private[:machine_init]`\n\n  `Ewebmachine.Builder.Handlers` `:add_handler` plug allows you to\n  set these parameters in order to use this Plug.\n\n  A successfull run will reset the resource handlers and initial state.","ref":"Ewebmachine.Plug.Run.html","title":"Ewebmachine.Plug.Run","type":"module"},{"doc":"Calling this plug will send the response and halt the connection\npipeline if the `conn` has passed through an `Ewebmachine.Plug.Run`.","ref":"Ewebmachine.Plug.Send.html","title":"Ewebmachine.Plug.Send","type":"module"},{"doc":"This plug checks the current response status. If it is an error, raise a plug\nexception with the status code and the HTTP error name as the message. If\nthis response body is not void, use it as the exception message.","ref":"Ewebmachine.Plug.ErrorAsException.html","title":"Ewebmachine.Plug.ErrorAsException","type":"exception"},{"doc":"","ref":"Ewebmachine.Plug.ErrorAsException.html#call/2","title":"Ewebmachine.Plug.ErrorAsException.call/2","type":"function"},{"doc":"","ref":"Ewebmachine.Plug.ErrorAsException.html#init/1","title":"Ewebmachine.Plug.ErrorAsException.init/1","type":"function"},{"doc":"# Changelog\n\n## [2.3.2] (2023-08-04)\n\n#","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"* Corrected the declaration of routes with plug when conditionaly removing declared resource.\n  (eg. Surrounding a `resource` with an if.)\n\n## [2.3.1] (2021-12-17)\n\n#","ref":"changelog.html#changed","title":"Changed - Changelog","type":"extras"},{"doc":"* `Ewebmachine.Plug.Send.call` now threads the `conn` struct to keep the updates instead of reusing the original one.\n* `Ewebmachine.Builder.Handles.add.add_handlers/2`: internal changes to satisfy dialyzer.","ref":"changelog.html#changed","title":"Changed - Changelog","type":"extras"},{"doc":"* Code janitoring wrt elixir versions: supports 1.3 - 1.6\n* Validated against Cowboy 1 & 2","ref":"changelog.html#v2-2","title":"v2.2 - Changelog","type":"extras"},{"doc":"* Fix\n  * Compilation with Plulg > 1.5 (https://github.com/kbrw/ewebmachine/issues/38)","ref":"changelog.html#v2-1-5","title":"v2.1.5 - Changelog","type":"extras"},{"doc":"* Feature: allow to name resources modules","ref":"changelog.html#v2-1-4","title":"v2.1.4 - Changelog","type":"extras"},{"doc":"* Bug fixes\n  * make it compatible with elixir 1.4 without warnings\n  * fix related regression","ref":"changelog.html#v2-1-3","title":"v2.1.3 - Changelog","type":"extras"},{"doc":"* Bug fixes\n  * Plug version spec and plug update to 1.0","ref":"changelog.html#v2-0-9","title":"v2.0.9 - Changelog","type":"extras"},{"doc":"* Bug fixes\n  * `accept_helper` is used in non terminal decision, so halt conn in case of 415 (else in some case, 415 became 200)","ref":"changelog.html#v2-0-8","title":"v2.0.8 - Changelog","type":"extras"},{"doc":"* Bug fixes\n  * All headers must be lower case to respect Plug.conn convention, updated tests\n  * add PATCH to default known methods, currently no other support, so to use PATCH\n    - set Accept-Patch in `option` with accepted media types, \n    - set handler for these types in `content-types-accepted`\n    - implement `resources_exists` to convert `PATCH` method to `PUT` if the resource exists","ref":"changelog.html#v2-0-7","title":"v2.0.7 - Changelog","type":"extras"},{"doc":"* Bug fixes\n  * Debug call log size were O(n^2), make it O(n) removing `conn.private` from\n    the log","ref":"changelog.html#v2-0-6","title":"v2.0.6 - Changelog","type":"extras"},{"doc":"* Enhancements\n  * `{:halt,code}` works in more cases because it now `throws` the `conn` to\n    break the decision flow.\n  * Make it possible to use `Ewebmachine.Plug.Debug` not at the routing root,\n    with relative assets. Thanks to @yrashk.\n  * Make it possible to use a fuzzy `content_types_accepted` media type. \n    Thanks again to @yrashk.\n  * little changes to allow to chain ewebmachine handler definitions and run\n  * 2 nice plugs to handle errors after a run : `ErrorAsException` and `ErrorAsForward`\n  * A macro to set common plug pipeline use cases : `resources_plugs`\n  * Change Logging from a simple Agent to ETS\n  * add this CHANGELOG\n\n* Bug fixes\n  * Bug in create path relative handling, use Conn.full_path to use the\n    `script_name` and the `path_info`","ref":"changelog.html#v2-0-5","title":"v2.0.5 - Changelog","type":"extras"},{"doc":"* Bug fixes\n  * Avoid module name collision with resource module function changing the\n    naming scheme.","ref":"changelog.html#v2-0-4","title":"v2.0.4 - Changelog","type":"extras"},{"doc":"* Bug fixes\n  * `Ewebmachine.Plug.Run` should run only if the `:machine_init` option has\n    been set.","ref":"changelog.html#v2-0-3","title":"v2.0.3 - Changelog","type":"extras"},{"doc":"* Enhancements\n  * Add `:default_plugs` option to `Ewebmachine.Builder.Resources`.","ref":"changelog.html#v2-0-2","title":"v2.0.2 - Changelog","type":"extras"},{"doc":"* Bug fixes\n  * Makes `{:halt,code}` management works correctly, add test.","ref":"changelog.html#v2-0-1","title":"v2.0.1 - Changelog","type":"extras"},{"doc":"* Bug fixes\n  * Makes `{:halt,code}` management works correctly.\n\n\n[2.3.1]: https://github.com/kbrw/ewebmachine/compare/v2.3.0...v2.3.1","ref":"changelog.html#v2-0-1","title":"v2.0.1 - Changelog","type":"extras"},{"doc":"Ewebmachine is a full rewrite with clean DSL and plug integration\nbased on Webmachine from basho. This version is not backward compatible with\nthe previous one that was only a thin wrapper around webmachine, use the branch\n1.0-legacy to use the old one.\n\n[![Build Status](https://travis-ci.org/kbrw/ewebmachine.svg?branch=master)](https://travis-ci.org/kbrw/ewebmachine)\n\nSee the [generated documentation](http://hexdocs.pm/ewebmachine) for more detailed explanations.\n\nThe principle is to go through the [HTTP decision tree](https://raw.githubusercontent.com/kbrw/ewebmachine/master/doc/http_diagram.png)\nand make decisions according to response of some callbacks called \"handlers\".\n\nTo do that, the library gives you 5 plugs and 2 plug pipeline builders :\n\n- `Ewebmachine.Plug.Run` go through the HTTP decision tree and fill\n  the `conn` response according to it\n- `Ewebmachine.Plug.Send` is used to send a conn set with `Ewebmachine.Plug.Run`\n- `Ewebmachine.Plug.Debug` gives you a debugging web UI to see the\n  HTTP decision path taken by each request.\n- `Ewebmachine.Plug.ErrorAsException` take a conn with a response set but not\n  send, and throw an exception is the status code is an exception\n- `Ewebmachine.Plug.ErrorAsForward` take a conn with a response set but not\n  send, and forward it changing the request to `GET /error/pattern/:status`\n- `Ewebmachine.Builder.Handlers` gives you helpers macros and a\n  `:add_handler` plug to add `handlers` as defined  in\n  `Ewebmachine.Handlers` to your conn, and set the initial user state.\n- `Ewebmachine.Builder.Resources` gives you a `resource` macro to\n  define at the same time an `Ewebmachine.Builder.Handlers` and the\n  matching spec to use it, and a plug `:resource_match` to do the\n  match and execute the associated plug. The macro `resources_plugs` helps you\n  to define commong plug pipeling","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"```elixir\ndefmodule MyJSONApi do \n  use Ewebmachine.Builder.Handlers\n  plug :cors\n  plug :add_handlers, init: %{}\n\n  content_types_provided do: [\"application/json\": :to_json]\n  defh to_json, do: Poison.encode!(state[:json_obj])\n\n  defp cors(conn,_), do: \n    put_resp_header(conn,\"Access-Control-Allow-Origin\",\"*\")\nend\n\n\ndefmodule ErrorRoutes do\n  use Ewebmachine.Builder.Resources ; resources_plugs\n  resource \"/error/:status\" do %{s: elem(Integer.parse(status),0)} after \n    content_types_provided do: ['text/html': :to_html, 'application/json': :to_json]\n    defh to_html, do: \"  Error ! : '#{Ewebmachine.Core.Utils.http_label(state.s)}' \"\n    defh to_json, do: ~s/{\"error\": #{state.s}, \"label\": \"#{Ewebmachine.Core.Utils.http_label(state.s)}\"}/\n    finish_request do: {:halt,state.s}\n  end\nend\n\ndefmodule FullApi do\n  use Ewebmachine.Builder.Resources\n  if Mix.env == :dev, do: plug Ewebmachine.Plug.Debug\n  resources_plugs error_forwarding: \"/error/:status\", nomatch_404: true\n  plug ErrorRoutes\n\n  resource \"/hello/:name\" do %{name: name} after \n    content_types_provided do: ['application/xml': :to_xml]\n    defh to_xml, do: \"  #{state.name} \"\n  end\n\n  resource \"/hello/json/:name\" do %{name: name} after \n    plug MyJSONApi #this is also a plug pipeline\n    allowed_methods do: [\"GET\",\"DELETE\"]\n    resource_exists do: pass((user=DB.get(state.name)) !== nil, json_obj: user)\n    delete_resource do: DB.delete(state.name)\n  end\n\n  resource \"/static/*path\" do %{path: Enum.join(path,\"/\")} after\n    resource_exists do:\n      File.regular?(path state.path)\n    content_types_provided do:\n      [{state.path|>Plug.MIME.path|>default_plain,:to_content}]\n    defh to_content, do:\n      File.stream!(path(state.path),[],300_000_000)\n    defp path(relative), do: \"#{:code.priv_dir :ewebmachine_example}/web/#{relative}\"\n    defp default_plain(\"application/octet-stream\"), do: \"text/plain\"\n    defp default_plain(type), do: type\n  end\nend\n```","ref":"readme.html#example-usage","title":"Example usage - Overview","type":"extras"},{"doc":"Go to `/wm_debug` to see precedent requests and debug there HTTP\ndecision path. The debug UI can be updated automatically on the\nrequests.\n\n![Debug UI example](https://raw.githubusercontent.com/kbrw/ewebmachine/master/doc/debug_ui.png)","ref":"readme.html#debug-ui","title":"Debug UI - Overview","type":"extras"},{"doc":"Create a simple supervision tree with only the Cowboy server adapter spec.\n\n```elixir\ndefmodule MyApp do\n  use Application\n  def start(_type, _args), do:\n    Supervisor.start_link([\n        Plug.Adapters.Cowboy.child_spec(:http,FullApi,[], port: 4000)\n      ], strategy: :one_for_one)\nend\n```\n\nAnd add it as your application entry point in your `mix.exs`\n\n```elixir\ndef application do\n  [applications: [:logger,:ewebmachine,:cowboy], mod: {MyApp,[]}]\nend\ndefp deps, do:\n  [{:ewebmachine, \"2.0.0\"}, {:cowboy, \"~> 1.0\"}]\n```\n\n# CONTRIBUTING\n\nHi, and thank you for wanting to contribute.\nPlease refer to the centralized informations available at: https://github.com/kbrw#contributing","ref":"readme.html#use-cowboy-to-serve-the-plug","title":"Use Cowboy to serve the plug - Overview","type":"extras"}]}